==> date_ical-4.0.0.tar.gz: date_ical/src/Plugin/Field/FieldFormatter/DateIcalFormatter.php <==
<?php

declare(strict_types = 1);

namespace Drupal\date_ical\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\datetime\Plugin\Field\FieldFormatter\DateTimeDefaultFormatter;
use Drupal\field\FieldConfigInterface;

/**
 * Plugin implementation of the 'Date iCal' formatter.
 *
 * @FieldFormatter(
 *   id = "date_ical",
 *   label = @Translation("Date iCal"),
 *   field_types = {
 *     "daterange",
 *     "datetime",
 *     "timestamp",
 *     "smartdate",
 *     "published_at"
 *   }
 * )
 */
final class DateIcalFormatter extends DateTimeDefaultFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $setting = [
      'summary_field' => '',
      'description_field' => '',
      'location_field' => '',
      'categories_field' => '',
      'organizer_field' => '',
      'status_field' => '',
      'url_field' => '',
      'attendee_field' => '',
      'alarm_field' => '',
      'attach_field' => '',
      'geo_field' => '',
      'disable_webcal' => FALSE,
      'download_directly' => TRUE,
    ];
    return $setting + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $elements = parent::settingsForm($form, $form_state);
    $textType = [
      'text',
      'text_long',
      'text_with_summary',
      'string',
      'string_long',
      'list_string',
    ];
    $entity_type = $this->fieldDefinition->getTargetEntityTypeId();
    $bundle = $this->fieldDefinition->getTargetBundle();
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $fieldDefinitions = $entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $listOption = [];
    foreach ($fieldDefinitions as $field_name => $field) {
      if ($field instanceof FieldConfigInterface) {
        $fieldType = $field->getType();
        $listOption[$fieldType][$field_name] = $field->getLabel();
      }
    }
    $elements['summary_field'] = [
      '#type' => 'select',
      '#title' => $this->t('SUMMARY field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->getSetting('summary_field'),
      '#description' => $this->t('You may optionally change the SUMMARY component for each event in the iCal output.
        Choose which text, taxonomy term reference or Node Reference field you would like to be output as the SUMMARY.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $elements['description_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Description field'),
      '#description' => $this->t("The views field to use as the body text for each event (DESCRIPTION).<br>
          If you wish to include another field you can leave this field blank it will render the row."),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->getSetting('description_field'),
    ];
    $elements['location_field'] = [
      '#type' => 'select',
      '#title' => $this->t('LOCATION field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType + ['ad' => 'address'], $listOption),
      '#default_value' => $this->getSetting('location_field'),
      '#description' => $this->t('You may optionally include a LOCATION component for each event in the iCal output.
        Choose which text or Node Reference field you would like to be output as the LOCATION.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $elements['geo_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Geo field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the geolocation for each event (GEO).'),
      '#options' => $this->getConfigurableFields([
        'geofield',
        'geolocation',
      ], $listOption),
      '#default_value' => $this->getSetting('geo_field'),
    ];
    $elements['categories_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Categories field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the categories for each event (CATEGORIES).'),
      '#options' => $this->getConfigurableFields([
        'entity_reference',
        'list_integer',
        'list_string',
      ], $listOption),
      '#default_value' => $this->getSetting('categories_field'),
    ];
    $elements['organizer_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Organizer field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the Organizer for each event (ORGANIZER).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->getSetting('organizer_field'),
    ];
    $elements['attendee_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Participant field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the attendee for each event (ATTENDEE).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->getSetting('attendee_field'),
    ];
    $elements['status_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Status field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the status for each event (STATUS), It must be empty or CONFIRMED,CANCELLED,TENTATIVE. Another value will crash'),
      '#options' => $this->getConfigurableFields(['list_string', 'list_states'], $listOption),
      '#default_value' => $this->getSetting('status_field'),
    ];
    $elements['url_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Url field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the URL for each event (URL).'),
      '#options' => $this->getConfigurableFields(['link', 'entity_reference'], $listOption),
      '#default_value' => $this->getSetting('url_field'),
    ];
    $elements['alarm_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Alarm field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) If it is an integer field, it means the number of minutes that will be reported.'),
      '#options' => $this->getConfigurableFields([
        'integer',
        'list_integer',
        'duration',
      ], $listOption),
      '#default_value' => $this->getSetting('alarm_field'),
    ];
    $elements['attach_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Attach field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the attach for each event (ATTACH).'),
      '#options' => $this->getConfigurableFields([
        'link',
        'file',
        'image',
        'entity_reference',
      ], $listOption),
      '#default_value' => $this->getSetting('attach_field'),
    ];
    $elements['disable_webcal'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disable webcal://'),
      '#default_value' => $this->getSetting('disable_webcal'),
      '#description' => $this->t("By default, the feed URL will use the webcal:// scheme, which allows calendar
        clients to easily subscribe to the feed. If you want your users to instead download this iCal
        feed as a file, activate this option."),
    ];
    $elements['download_directly'] = [
      '#type' => 'checkbox',
      '#parents' => ['row_options', 'download_directly'],
      '#title' => $this->t('Download directly'),
      '#description' => $this->t('Export the iCal in plain text.'),
      '#default_value' => $this->getSetting('download_directly'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    foreach ($this->getSettings() as $key => $setting) {
      $summary[$key] = ucwords(str_replace('_', ' ', $key)) . ': ' . $this->getSetting($key);
    }
    return $summary;
  }

  /**
   * Get list of fields.
   */
  protected function getConfigurableFields($type = FALSE, $listFields = []) {
    $resultField = [];
    foreach ($listFields as $field_name => $fields) {
      if (in_array($field_name, $type)) {
        $resultField += $fields;
      }
    }
    return $resultField;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $element = $items->view();
    $view_mode = $this->viewMode;
    if ($view_mode == 'full') {
      $view_mode = 'default';
    }
    $entity = $items->getEntity();
    $url = Url::fromRoute('date_ical.field', [
      'entity_type' => $this->fieldDefinition->getTargetEntityTypeId(),
      'entity_id' => $entity->id(),
      'field_name' => $items->getName(),
      'view_mode' => $view_mode,
    ])->setAbsolute()->toString();
    if (!$this->getSetting('disable_webcal')) {
      $url = str_replace(['http://', 'https://'], 'webcal://', $url);
    }
    $element[0] = [
      '#theme' => 'date_ical_icon',
      '#url' => $url,
      '#title' => $element[0],
      '#format' => 'text/calendar',
      '#attributes' => [
        'class' => [
          'ical-icon',
        ],
        'download' => 'event.ics',
      ],
    ];
    return $element;
  }

}

==> date_ical-4.0.1.tar.gz: date_ical/src/Plugin/Field/FieldFormatter/DateIcalFormatter.php <==
<?php

declare(strict_types = 1);

namespace Drupal\date_ical\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\datetime\Plugin\Field\FieldFormatter\DateTimeDefaultFormatter;

/**
 * Plugin implementation of the 'Date iCal' formatter.
 *
 * @FieldFormatter(
 *   id = "date_ical",
 *   label = @Translation("Date iCal"),
 *   field_types = {
 *     "daterange",
 *     "datetime",
 *     "timestamp",
 *     "smartdate",
 *     "published_at"
 *   }
 * )
 */
final class DateIcalFormatter extends DateTimeDefaultFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $setting = [
      'summary_field' => '',
      'description_field' => '',
      'location_field' => '',
      'categories_field' => '',
      'organizer_field' => '',
      'status_field' => '',
      'url_field' => '',
      'attendee_field' => '',
      'alarm_field' => '',
      'attach_field' => '',
      'geo_field' => '',
      'disable_webcal' => TRUE,
      'download_directly' => TRUE,
    ];
    return $setting + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $elements = parent::settingsForm($form, $form_state);
    $textType = [
      'text',
      'text_long',
      'text_with_summary',
      'string',
      'string_long',
      'list_string',
    ];
    $entity_type = $this->fieldDefinition->getTargetEntityTypeId();
    $bundle = $this->fieldDefinition->getTargetBundle();
    $entityFieldManager = \Drupal::service('entity_field.manager');
    $fieldDefinitions = $entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $listOption = [];
    foreach ($fieldDefinitions as $field_name => $field) {
      $fieldType = $field->getType();
      $listOption[$fieldType][$field_name] = $field->getLabel();
    }
    $elements['summary_field'] = [
      '#type' => 'select',
      '#title' => $this->t('SUMMARY field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->getSetting('summary_field'),
      '#description' => $this->t('You may optionally change the SUMMARY component for each event in the iCal output.
        Choose which text, taxonomy term reference or Node Reference field you would like to be output as the SUMMARY.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $elements['description_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Description field'),
      '#description' => $this->t("The views field to use as the body text for each event (DESCRIPTION).<br>
          If you wish to include another field you can leave this field blank it will render the row."),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType, $listOption),
      '#default_value' => $this->getSetting('description_field'),
    ];
    $elements['location_field'] = [
      '#type' => 'select',
      '#title' => $this->t('LOCATION field'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getConfigurableFields($textType + [
        'ad' => 'address',
        'er' => 'entity_reference',
      ], $listOption),
      '#default_value' => $this->getSetting('location_field'),
      '#description' => $this->t('You may optionally include a LOCATION component for each event in the iCal output.
        Choose which text or Node Reference field you would like to be output as the LOCATION.
        If using a Node Reference, the Title of the referenced node will be used.'),
    ];
    $elements['geo_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Geo field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the geolocation for each event (GEO).'),
      '#options' => $this->getConfigurableFields([
        'geofield',
        'geolocation',
      ], $listOption),
      '#default_value' => $this->getSetting('geo_field'),
    ];
    $elements['categories_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Categories field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the categories for each event (CATEGORIES).'),
      '#options' => $this->getConfigurableFields([
        'entity_reference',
        'list_integer',
        'list_string',
      ], $listOption),
      '#default_value' => $this->getSetting('categories_field'),
    ];
    $elements['organizer_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Organizer field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the Organizer for each event (ORGANIZER).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->getSetting('organizer_field'),
    ];
    $elements['attendee_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Participant field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field (email or user) to use as the attendee for each event (ATTENDEE).'),
      '#options' => $this->getConfigurableFields(['entity_reference', 'email'], $listOption),
      '#default_value' => $this->getSetting('attendee_field'),
    ];
    $elements['status_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Status field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the status for each event (STATUS), It must be empty or CONFIRMED,CANCELLED,TENTATIVE. Another value will crash'),
      '#options' => $this->getConfigurableFields(['list_string', 'list_states'], $listOption),
      '#default_value' => $this->getSetting('status_field'),
    ];
    $elements['url_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Url field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the URL for each event (URL).'),
      '#options' => $this->getConfigurableFields(['link', 'entity_reference'], $listOption),
      '#default_value' => $this->getSetting('url_field'),
    ];
    $elements['alarm_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Alarm field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) If it is an integer field, it means the number of minutes that will be reported.'),
      '#options' => $this->getConfigurableFields([
        'integer',
        'list_integer',
        'duration',
      ], $listOption),
      '#default_value' => $this->getSetting('alarm_field'),
    ];
    $elements['attach_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Attach field'),
      '#empty_option' => $this->t('- None -'),
      '#description' => $this->t('(optional) The views field to use as the attach for each event (ATTACH).'),
      '#options' => $this->getConfigurableFields([
        'link',
        'file',
        'image',
        'entity_reference',
      ], $listOption),
      '#default_value' => $this->getSetting('attach_field'),
    ];
    $elements['disable_webcal'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Disable webcal://'),
      '#default_value' => $this->getSetting('disable_webcal'),
      '#description' => $this->t("By default, the feed URL will use the webcal:// scheme, which allows calendar
        clients to easily subscribe to the feed. If you want your users to instead download this iCal
        feed as a file, activate this option."),
    ];
    $elements['download_directly'] = [
      '#type' => 'checkbox',
      '#parents' => ['row_options', 'download_directly'],
      '#title' => $this->t('Download directly'),
      '#description' => $this->t('Export the iCal in plain text.'),
      '#default_value' => $this->getSetting('download_directly'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    foreach ($this->getSettings() as $key => $setting) {
      $summary[$key] = ucwords(str_replace('_', ' ', $key)) . ': ' . $this->getSetting($key);
    }
    return $summary;
  }

  /**
   * Get list of fields.
   */
  protected function getConfigurableFields($type = FALSE, $listFields = []) {
    $resultField = [];
    foreach ($listFields as $field_name => $fields) {
      if (in_array($field_name, $type)) {
        $resultField += $fields;
      }
    }
    return $resultField;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $element = $items->view();
    $view_mode = $this->viewMode;
    if ($view_mode == 'full') {
      $view_mode = 'default';
    }
    $entity = $items->getEntity();
    $url = Url::fromRoute('date_ical.field', [
      'entity_type' => $this->fieldDefinition->getTargetEntityTypeId(),
      'entity_id' => $entity->id(),
      'field_name' => $items->getName(),
      'view_mode' => $view_mode,
    ])->setAbsolute()->toString();
    if (!$this->getSetting('disable_webcal')) {
      $url = str_replace(['http://', 'https://'], 'webcal://', $url);
    }
    $element[0] = [
      '#theme' => 'date_ical_icon',
      '#url' => $url,
      '#title' => $element[0],
      '#format' => 'text/calendar',
      '#attributes' => [
        'class' => [
          'ical-icon',
        ],
        'download' => 'event.ics',
      ],
    ];
    return $element;
  }

}
